/*! \addtogroup registrar Registrar
    @{
*/

#ifndef Keeper_cpp
#define Keeper_cpp

#include <czmq.h>
#include <json/json.h>
#include <map>
#include <string>

using namespace std;

/*!
	\brief This class accumulates data.
    \author Mayorov

	Receives data from 'Subscriber' and store. After a specified period of time, obtains an average value of accumulated data and sends it on.
*/
class Keeper {
public:
    Keeper();
    virtual ~Keeper();
    
    /*!
        Receives messages and writes to log.
        \param[in] message Message from broker. Pair<type of node, JSON>
    */
    void Insert_Message(pair<string, string> message);
    
    /*!
        Sets length of time over which data should be communicated.
        \param[in] time Time interval in milliseconds
    */
    void Set_Time_Calc(int time);
    
    /*!
        Informs whether to transmit data. Check for the expiration of the period.
        \return Returns a Boolean variable. True or false.
    */
    bool Time_Write();
    
    /*!
        Returns map with the average load of the cluster for certain period of time.
        \return Returns a map with the data. map<pair<[type of node], [hostname]>, map<[name of parameter], [value]>>
    */
    map<pair<string, string>, map<string, double> > Get_Map();
    
    /*!
        Clears log with accumulated data.
    */
    void Clear_Map();

private:
    /*!
        Object JSON
    */
    Json::Value root;
    
    /*!
        JSON parser
    */
    Json::Reader reader;
    
    /*!
        Log data
    */
    map<pair<string, string>, map<string, double> > map_journal;
    
    /*!
        Last transfer time
    */
    int64_t time;
    
    /*!
        The period through which should pass journal
    */
    int time_calculate;
};

#endif

/*! @} */