/*! 
    \defgroup registrar Registrar
    \brief This module writes a story download o cluster in database.
    
     The module receives data from broker, calculates average value over a certain period of time and writes to database. Data is written over a user-defined period, day and month.
*/
///@{

#ifndef Subscriber_cpp
#define Subscriber_cpp

#include <czmq.h>
#include <string>

using namespace std;

/*!
	\brief This class receives data from the broker.
    \author Mayorov

	The class receives data from the broker and passes it out. The data are stored only until a new message is received.
*/
class Subscriber {
public:
    /*!
        The constructor creates class 'Subscriber' with the specified address
        \param[in] addr_sub Address of the broker
    */
    Subscriber(char* addr_sub);
    
    /*!
        The default constructor creates class 'Subscriber' with address 'tcp://localhost:5001'
    */
    Subscriber();
    virtual ~Subscriber();
    
    /*!
        Returns broker information about the cluster.
        \return Pair <[type of node], [json]>
    */
    pair<string, string> Get_Cluster_Json();
    
    /*!
        Waiting for a message from broker.
    */
    void Recv_Message();
    
    /*!
        Connects to the broker.
    */
    void Connect();
    
    /*!
        Sets address of broker
        \param[in] addr Address of the broker
        \warning Use before establishing connection with broker
    */
    void Set_Addr(char* addr);

private:
    /*!
        Address of the broker
    */
    char* addr_sub;
    
    /*!
        Socket ZeroMQ
    */
    zsock_t* sub;
    
    /*!
        Type of node
    */
    string cluster;
    
    /*!
        JSON
    */
    string json;
};

#endif

///@}