#include "Writer.h"
using namespace std;

Writer::Writer()
{
    server = strdup("localhost");
    user = strdup("root");
    password = strdup("");
    database = strdup("slurm_acct");

    write_day = true;
    write_month = true;
    time(&t);
    tk = localtime(&t);
    cur_day = tk->tm_mday;
    cur_month = tk->tm_mon;
}

Writer::~Writer()
{
    mysql_free_result(res);
    mysql_close(reader);
    mysql_close(writer);
    free(server);
    free(user);
    free(password);
    free(database);
}

void Writer::Set_Server(char* serv)
{
    server = serv;
}

void Writer::Set_User(char* usr)
{
    user = usr;
}

void Writer::Set_Password(char* psd)
{
    password = psd;
}

void Writer::Set_Database(char* db)
{
    database = db;
}

void Writer::Set_Off_Write_Day()
{
    write_day = false;
}

void Writer::Set_Off_Write_Month()
{
    write_month = false;
}

bool Writer::Connect_DB()
{
    writer = mysql_init(NULL);
    reader = mysql_init(NULL);
    
    if (!mysql_real_connect(writer, server, user, password, database, 0, NULL, 0)) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        return false;
    }

    if (!mysql_real_connect(reader, server, user, password, database, 0, NULL, 0)) {
        fprintf(stderr, "%s\n", mysql_error(reader));
        return false;
    }

    printf("Сonnection to database is established\n");
    return true;
}

void Writer::Check_Connection()
{
    if(mysql_ping(writer) > 0)
    {
        if (!mysql_real_connect(writer, server, user, password, database, 0, NULL, 0)) {
            fprintf(stderr, "%s\n", mysql_error(writer));
        }
    }
    
    if(mysql_ping(reader) > 0)
    {
        if (!mysql_real_connect(reader, server, user, password, database, 0, NULL, 0)) {
            fprintf(stderr, "%s\n", mysql_error(reader));
        }
    }
}

void Writer::Create_Tables()
{
    if (mysql_query(writer, "CREATE TABLE cluster_usage_hourly (id INT NOT NULL PRIMARY KEY AUTO_INCREMENT, type varchar(50), nodename varchar (100) NOT NULL, time BIGINT(15),  cpu_user DOUBLE (50, 2) NOT NULL, cpu_sys DOUBLE (50, 2) NOT NULL,  mem_total DOUBLE (50, 2) NOT NULL, mem_used DOUBLE (50, 2) NOT NULL, mem_buffer DOUBLE (50, 2) NOT NULL, mem_cached DOUBLE (50, 2) NOT NULL, net_in DOUBLE (50, 2) NOT NULL, net_out DOUBLE (50, 2) NOT NULL)")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    if (mysql_query(writer, "CREATE TABLE cluster_usage_daily (id INT NOT NULL PRIMARY KEY AUTO_INCREMENT, type varchar(50), nodename varchar (100) NOT NULL, time BIGINT(15),  cpu_user DOUBLE (50, 2) NOT NULL, cpu_sys DOUBLE (50, 2) NOT NULL,  mem_total DOUBLE (50, 2) NOT NULL, mem_used DOUBLE (50, 2) NOT NULL, mem_buffer DOUBLE (50, 2) NOT NULL, mem_cached DOUBLE (50, 2) NOT NULL, net_in DOUBLE (50, 2) NOT NULL, net_out DOUBLE (50, 2) NOT NULL)")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    if (mysql_query(writer, "CREATE TABLE cluster_usage_monthly (id INT NOT NULL PRIMARY KEY AUTO_INCREMENT, type varchar(50), nodename varchar (100) NOT NULL, time BIGINT(15),  cpu_user DOUBLE (50, 2) NOT NULL, cpu_sys DOUBLE (50, 2) NOT NULL,  mem_total DOUBLE (50, 2) NOT NULL, mem_used DOUBLE (50, 2) NOT NULL, mem_buffer DOUBLE (50, 2) NOT NULL, mem_cached DOUBLE (50, 2) NOT NULL, net_in DOUBLE (50, 2) NOT NULL, net_out DOUBLE (50, 2) NOT NULL)")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    printf("Tables are created\n");
}

void Writer::Clear_Tables()
{
    if (mysql_query(writer, "TRUNCATE TABLE cluster_usage_hourly")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    if (mysql_query(writer, "TRUNCATE TABLE cluster_usage_daily")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    if (mysql_query(writer, "TRUNCATE TABLE cluster_usage_monthly")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    printf("Tables are cleared\n");
}

void Writer::Write(map<pair<string, string>, map<string, double> > map_journal)
{
    Check_Connection();
    
    for (map<pair<string, string>, map<string, double> >::iterator iter = map_journal.begin(); iter != map_journal.end(); iter++) {
        string type = iter->first.first;
        string nodename = iter->first.second;
        double cpu_user = iter->second.find("cpu_user")->second;
        double cpu_sys = iter->second.find("cpu_sys")->second;
        double net_in = iter->second.find("net_in")->second;
        double net_out = iter->second.find("net_out")->second;
        double mem_total = iter->second.find("total")->second;
        double mem_used = iter->second.find("mem_used")->second;
        double mem_buffer = iter->second.find("buffer")->second;
        double mem_cached = iter->second.find("cached")->second;
        char* sql_query = zsys_sprintf("INSERT INTO cluster_usage_hourly (nodename, type, time, cpu_user, cpu_sys, mem_total, mem_used, mem_buffer, mem_cached, net_in, net_out) VALUES('%s', '%s', (UNIX_TIMESTAMP() * 1000), %f, %f, %f, %f, %f, %f, %f, %f)", nodename.data(), type.data(), cpu_user, cpu_sys, mem_total, mem_used, mem_buffer, mem_cached, net_in, net_out);
        if (mysql_query(writer, sql_query)) {
            fprintf(stderr, "%s\n", mysql_error(writer));
            exit(1);
        }
        free(sql_query);
    }

    if (write_day)
        Write_Day();
}

void Writer::Write_Day()
{
    time(&t);
    tk = localtime(&t);
    if (cur_day != tk->tm_mday) {
        if (mysql_query(reader, "SELECT type, nodename, AVG(cpu_user) as cpu_user, AVG(cpu_sys) AS cpu_sys, AVG(mem_total) AS mem_total, AVG(mem_used) AS mem_used, AVG(mem_buffer) AS mem_buffer, AVG(mem_cached) AS mem_cached, AVG(net_in) AS net_in, AVG(net_out) AS net_out FROM cluster_usage_hourly WHERE time >= UNIX_TIMESTAMP(DATE_SUB(CURDATE(), INTERVAL 1 DAY)) * 1000 AND time < UNIX_TIMESTAMP(CURDATE()) * 1000 GROUP BY type, nodename ORDER BY time DESC")) {
            fprintf(stderr, "%s\n", mysql_error(reader));
            exit(1);
        }

        res = mysql_use_result(reader);
        while ((row = mysql_fetch_row(res)) != NULL) {
            string type = row[0];
            string nodename = row[1];
            double cpu_user = atof(row[2]);
            double cpu_sys = atof(row[3]);
            double mem_total = atof(row[4]);
            double mem_used = atof(row[5]);
            double mem_buffer = atof(row[6]);
            double mem_cached = atof(row[7]);
            double net_in = atof(row[8]);
            double net_out = atof(row[9]);

            char* sql_query = zsys_sprintf("INSERT INTO cluster_usage_daily (nodename, type, time, cpu_user, cpu_sys, mem_total, mem_used, mem_buffer, mem_cached, net_in, net_out) VALUES('%s', '%s', (UNIX_TIMESTAMP() * 1000), %f, %f, %f, %f, %f, %f, %f, %f)", nodename.data(), type.data(), cpu_user, cpu_sys, mem_total, mem_used, mem_buffer, mem_cached, net_in, net_out);
            if (mysql_query(writer, sql_query)) {
                fprintf(stderr, "%s\n", mysql_error(writer));
                exit(1);
            }
            free(sql_query);
        }
        cur_day = tk->tm_mday;
        mysql_free_result(res);
    }

    if (write_month)
        Write_Month();
}

void Writer::Write_Month()
{
    time(&t);
    tk = localtime(&t);
    if (cur_month != tk->tm_mon) {
        if (mysql_query(reader, "SELECT type, nodename, AVG(cpu_user) as cpu_user, AVG(cpu_sys) AS cpu_sys, AVG(mem_total) AS mem_total, AVG(mem_used) AS mem_used, AVG(mem_buffer) AS mem_buffer, AVG(mem_cached) AS mem_cached, AVG(net_in) AS net_in, AVG(net_out) AS net_out FROM cluster_usage_daily WHERE time > UNIX_TIMESTAMP(LAST_DAY(DATE_SUB(CURDATE(), INTERVAL 2 MONTH))) * 1000 AND time < UNIX_TIMESTAMP(DATE_ADD(LAST_DAY(CURDATE() - INTERVAL 1 MONTH), INTERVAL 1 DAY)) * 1000 GROUP BY type, nodename ORDER BY time DESC")) {
            fprintf(stderr, "%s\n", mysql_error(reader));
            exit(1);
        }

        res = mysql_use_result(reader);
        while ((row = mysql_fetch_row(res)) != NULL) {
            string type = row[0];
            string nodename = row[1];
            double cpu_user = atof(row[2]);
            double cpu_sys = atof(row[3]);
            double mem_total = atof(row[4]);
            double mem_used = atof(row[5]);
            double mem_buffer = atof(row[6]);
            double mem_cached = atof(row[7]);
            double net_in = atof(row[8]);
            double net_out = atof(row[9]);

            char* sql_query = zsys_sprintf("INSERT INTO cluster_usage_monthly (nodename, type, time, cpu_user, cpu_sys, mem_total, mem_used, mem_buffer, mem_cached, net_in, net_out) VALUES('%s', '%s', (UNIX_TIMESTAMP() * 1000), %f, %f, %f, %f, %f, %f, %f, %f)", nodename.data(), type.data(), cpu_user, cpu_sys, mem_total, mem_used, mem_buffer, mem_cached, net_in, net_out);
            if (mysql_query(writer, sql_query)) {
                fprintf(stderr, "%s\n", mysql_error(writer));
                exit(1);
            }
            free(sql_query);
        }
        cur_month = tk->tm_mon;
        mysql_free_result(res);

        Delete_Old_Records();
    }
}

void Writer::Delete_Old_Records()
{
    if (mysql_query(writer, "DELETE FROM cluster_usage_hourly WHERE time < UNIX_TIMESTAMP(DATE_SUB(CURDATE(), INTERVAL 2 MONTH)) * 1000")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    if (mysql_query(writer, "DELETE FROM cluster_usage_daily WHERE time < UNIX_TIMESTAMP(DATE_SUB(CURDATE(), INTERVAL 3 YEAR)) * 1000")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }

    if (mysql_query(writer, "DELETE FROM cluster_usage_monthly WHERE time < UNIX_TIMESTAMP(DATE_SUB(CURDATE(), INTERVAL 5 YEAR)) * 1000")) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        exit(1);
    }
}