#include "HlitNodeSensor.h"
#include <sstream>
using namespace std;
HlitNodeSensor::HlitNodeSensor(char* net_list)
    : fCPU(0)
    , fMem(0)
    , fNetList(0)
    , fNetArray(0)
{
    string to_split = net_list;

    size_t pos = 0, found;
    while ((found = to_split.find_first_of(',', pos)) != string::npos) {
        NetVector.push_back(to_split.substr(pos, found - pos));
        pos = found + 1;
    }
    NetVector.push_back(to_split.substr(pos));
    vector<string>::iterator it = NetVector.begin();
    if (strcmp((*it).data(), "all") == 0) {
        NetVector.clear();
    }
}
HlitNodeSensor::~HlitNodeSensor()
{
    delete fCPU;
    delete fMem;

    if (fNetArray) {
        glibtop_netload* net = (glibtop_netload*)zlistx_first(fNetArray);
        for (unsigned int i = 0; i < fNetList->number; i++) {
            delete net;
            net = (glibtop_netload*)zlistx_next(fNetArray);
        }
        zlistx_destroy(&fNetArray);
    }
    delete fNetList;
}

void HlitNodeSensor::Init()
{
    fCPU = new glibtop_cpu;
    fMem = new glibtop_mem;
    fNetList = new glibtop_netlist;
    fNetArray = zlistx_new();

    char** devices = glibtop_get_netlist(fNetList);
    glibtop_netload* net;
    for (unsigned int i = 0; i < fNetList->number; i++) {
        net = new glibtop_netload;
        zlistx_add_end(fNetArray, net);
        free(devices[i]);
    }
    free(devices);
}

glibtop_cpu* HlitNodeSensor::GetCpu()
{
    return fCPU;
}

glibtop_mem* HlitNodeSensor::GetMem()
{
    return fMem;
}

glibtop_netlist* HlitNodeSensor::GetNetList()
{
    return fNetList;
}

zlistx_t* HlitNodeSensor::GetNetArray()
{
    return fNetArray;
}

int HlitNodeSensor::Update()
{
    if (!fCPU)
        Init();

    glibtop_get_cpu(fCPU);
    glibtop_get_mem(fMem);

    char** devices = glibtop_get_netlist(fNetList);
    glibtop_netload* net;

    net = (glibtop_netload*)zlistx_first(fNetArray);
    for (unsigned int i = 0; i < fNetList->number; i++) {
        //        printf("dev=%s\n", devices[i]);
        glibtop_get_netload(net, devices[i]);
        net = (glibtop_netload*)zlistx_next(fNetArray);
        free(devices[i]);
    }
    free(devices);

    return 0;
}

void HlitNodeSensor::GetSpeed(HlitNodeSensor* s1, HlitNodeSensor* s2, int timeout)
{

    if (!fCPU)
        Init();

    glibtop_cpu* cpu1 = s1->GetCpu();
    glibtop_cpu* cpu2 = s2->GetCpu();

    fCPU->total = (cpu2->total - cpu1->total);
    fCPU->sys = (cpu2->sys - cpu1->sys);
    fCPU->user = (cpu2->user - cpu1->user);
    fCPU->nice = (cpu2->nice - cpu1->nice);
    fCPU->idle = (cpu2->idle - cpu1->idle);
    fCPU->iowait = (cpu2->iowait - cpu1->iowait);
    fCPU->irq = (cpu2->irq - cpu1->irq);
    fCPU->softirq = (cpu2->softirq - cpu1->softirq);
    fCPU->frequency = (cpu2->frequency - cpu1->frequency);

    glibtop_mem* mem1 = s1->GetMem();
    glibtop_mem* mem2 = s2->GetMem();

    fMem->total = (mem2->total - mem1->total) * 1000 / timeout;
    fMem->used = (mem2->used - mem1->used) * 1000 / timeout;
    fMem->free = (mem2->free - mem1->free) * 1000 / timeout;
    fMem->shared = (mem2->shared - mem1->shared) * 1000 / timeout;
    fMem->buffer = (mem2->buffer - mem1->buffer) * 1000 / timeout;
    fMem->cached = (mem2->cached - mem1->cached) * 1000 / timeout;
    fMem->locked = (mem2->locked - mem1->locked) * 1000 / timeout;

    //  char** devices = glibtop_get_netlist(fNetList);
    glibtop_netload* net = (glibtop_netload*)zlistx_first(fNetArray);
    glibtop_netload* net1 = (glibtop_netload*)zlistx_first(s1->GetNetArray());
    glibtop_netload* net2 = (glibtop_netload*)zlistx_first(s2->GetNetArray());
    for (unsigned int i = 0; i < fNetList->number; i++) {

        if (net) {
            net->address = (net2->address - net1->address) * 1000 / timeout;
            net->bytes_in = (net2->bytes_in - net1->bytes_in) * 1000 / timeout;
            net->bytes_out = (net2->bytes_out - net1->bytes_out) * 1000 / timeout;
            net->errors_in = (net2->errors_in - net1->errors_in) * 1000 / timeout;
            net->errors_out = (net2->errors_out - net1->errors_out) * 1000 / timeout;
            net->packets_in = (net2->packets_in - net1->packets_in) * 1000 / timeout;
            net->packets_out = (net2->packets_out - net1->packets_out) * 1000 / timeout;
        }

        net = (glibtop_netload*)zlistx_next(fNetArray);
        net1 = (glibtop_netload*)zlistx_next(s1->GetNetArray());
        net2 = (glibtop_netload*)zlistx_next(s2->GetNetArray());
    }
}

bool HlitNodeSensor::FindAdapter(vector<string> vctr, string adapter) const
{
    for (vector<string>::iterator it = vctr.begin(); it != vctr.end(); ++it) {
        if (strcmp((*it).data(), adapter.data()) == 0)
            return true;
    }
    return false;
}

/* returning JSON data in char* type string */
char* HlitNodeSensor::GetJson(const char* name, bool debug) const
{

    string json;
    double precent = (double)fCPU->total;

    // start ROOT
    json += "\"";
    json += name;
    json += "\" : {";

    char* cpu_str = zsys_sprintf("\"cpu\" : { "
                                 "\"total\" : %.3f, "
                                 "\"sys\" : %.3f, "
                                 "\"user\" : %.3f, "
                                 "\"nice\" : %.3f, "
                                 "\"idle\" : %.3f, "
                                 "\"iowait\" : %.3f, "
                                 "\"irq\" : %.3f, "
                                 "\"softirq\" : %.3f, "
                                 "\"frequency\" : %.3f "
                                 "}",
        ((double)fCPU->total / precent) * 100.0, ((double)fCPU->sys / precent) * 100.0, ((double)fCPU->user / precent) * 100.0, ((double)fCPU->nice / precent) * 100.0, ((double)fCPU->idle / precent) * 100.0,
        ((double)fCPU->iowait / precent) * 100.0, ((double)fCPU->irq / precent) * 100.0, ((double)fCPU->softirq / precent) * 100.0, (double)fCPU->frequency);
    json += cpu_str;
    free(cpu_str);
    //   return json.data();

    // mem
    json += ", ";
    char* mem_str = zsys_sprintf("\"mem\" : { "
                                 "\"total\" : %lld, "
                                 "\"used\" : %lld, "
                                 "\"free\" : %lld, "
                                 "\"shared\" : %lld, "
                                 "\"buffer\" : %lld, "
                                 "\"cached\" : %lld, "
                                 "\"locked\" : %lld "
                                 "}",
        fMem->total, fMem->used, fMem->free, fMem->shared, fMem->buffer,
        fMem->cached, fMem->locked);
    json += mem_str;
    free(mem_str);

    // net
    json += ", ";
    json += "\"net\" : [";
    char** devices = glibtop_get_netlist(fNetList);
    glibtop_netload* net = (glibtop_netload*)zlistx_first(fNetArray);
    int count = 0;
    for (unsigned int i = 0; i < fNetList->number; i++) {
        if (net && (NetVector.size() == 0 || FindAdapter(NetVector, devices[i]) == true)) {
            if (count != 0)
                json += ",";
            char* net_str = zsys_sprintf("{\"adapter\" : \"%s\", "
                                         "\"address\" : %lld, "
                                         "\"bytes_in\" : %lld, "
                                         "\"bytes_out\" : %lld, "
                                         "\"errors_in\" : %lld, "
                                         "\"errors_out\" : %lld, "
                                         "\"packets_in\" : %lld, "
                                         "\"packets_out\" : %lld ",
                devices[i], net->address, net->bytes_in, net->bytes_out,
                net->errors_in, net->errors_out, net->packets_in, net->packets_out);
            json += net_str;
            json += "}";
            count++;
            free(net_str);
        }
        free(devices[i]);
        net = (glibtop_netload*)zlistx_next(fNetArray);
    }
    free(devices);

    json += "]";

    //     end ROOT
    json += "}";

    if (debug)
        printf("\n %s", json.data());
    return zsys_sprintf("%s", json.data());
}

void HlitNodeSensor::Print() const
{

    printf("Cpu Data :  \n");
    printf("  total : %ld \n", fCPU->total);
    printf("  user : %ld \n", fCPU->user);
    printf("  nice : %ld \n", fCPU->nice);
    printf("  sys: %ld \n", fCPU->sys);
    printf("  idle : %ld \n", fCPU->idle);
    printf("  iowait : %ld \n", fCPU->iowait);
    printf("  irq : %ld \n", fCPU->irq);
    printf("  softirq : %ld \n", fCPU->softirq);
    printf("  frequency : %ld \n", fCPU->frequency);

    printf("Memory Data :  \n");
    printf("  total : %ld \n", fMem->total);
    printf("  used : %ld \n", fMem->used);
    printf("  free : %ld \n", fMem->free);
    printf("  shared : %ld \n", fMem->shared);
    printf("  buffer : %ld \n", fMem->buffer);
    printf("  cached : %ld \n", fMem->cached);
    printf("  user : %ld \n", fMem->user);
    printf("  locked : %ld \n", fMem->locked);

    printf("Net Data :  %d devices\n", fNetList->number);
    char** devices = glibtop_get_netlist(fNetList);
    glibtop_netload* net = (glibtop_netload*)zlistx_first(fNetArray);
    for (unsigned int i = 0; i < fNetList->number; i++) {
        if (net && (NetVector.size() == 0 || FindAdapter(NetVector, devices[i]))) {
            printf("  %s\n", devices[i]);
            printf("    address : %d\n", net->address);
            printf("    bytes_in : %ld\n", net->bytes_in);
            printf("    bytes_out : %ld\n", net->bytes_out);
            printf("    errors_in : %ld\n", net->errors_in);
            printf("    errors_out : %ld\n", net->errors_out);
            printf("    packets_in : %ld\n", net->packets_in);
            printf("    packets_out : %ld\n", net->packets_out);
        }
        free(devices[i]);
        net = (glibtop_netload*)zlistx_next(fNetArray);
    }
    free(devices);
}
