/*! 
    \defgroup sensor Sensor
    \brief This module receives data from the computer.
    
     The module retrieves data about CPU, memory and network.
*/
///@{

#ifndef HlitNodeSensor_cpp
#define HlitNodeSensor_cpp

#include <czmq.h>
#include <glibtop/cpu.h>
#include <glibtop/mem.h>
#include <glibtop/netlist.h>
#include <glibtop/netload.h>
#include <string>
#include <vector>

class HlitNodeSensor {
public:
    HlitNodeSensor(char* net_list);
    virtual ~HlitNodeSensor();

    void Init();
    int Update();
    char* GetJson(const char* name = "static", bool debug = false) const;
    void Print() const;
    void GetSpeed(HlitNodeSensor* s1, HlitNodeSensor* s2, int timeout);
    bool FindAdapter(std::vector<std::string> vctr, std::string adapter) const;

    glibtop_cpu* GetCpu();
    glibtop_mem* GetMem();
    glibtop_netlist* GetNetList();
    zlistx_t* GetNetArray();

private:
    glibtop_cpu* fCPU;
    glibtop_mem* fMem;
    glibtop_netlist* fNetList;
    zlistx_t* fNetArray;
    std::vector<std::string> NetVector;
};

#endif

///@}
