#include "../registrar/Keeper.h"
#include "../registrar/Subscriber.h"
#include "../registrar/Writer.h"
#include <czmq.h>
#include <getopt.h>

void help(char* progname)
{
    printf("\n%s - connects to broker and receives data of specified time. Then finds average for each parameter and makes records in the database\n", progname);
    printf("\nUsage: %s <command>\n", progname);
    printf("\nWhere <command> is:\n");
    printf("\t--time_calculate \t [-t] <milliseconds> \t The period of time that program collects data and makes records in database\n");
    printf("\t--subscribe \t\t [-s] <address> \t Address, where messages are coming\n");
    printf("\t--server \t\t [-e] <name> \t\t Server on which the database resides\n");
    printf("\t--user \t\t\t [-u] <name> \t\t The user on whose behalf program makes records in the database\n");
    printf("\t--password \t\t [-p] <password> \t The password of user on whose behalf program makes records in the database\n");
    printf("\t--database \t\t [-d] <name> \t\t The name of database to which program makes records\n");
    printf("\t--create_table \t\t [-c] \t\t\t The program creates tables 'cluster_usage_hourly', 'cluster_usage_daily', 'cluster_usage_monthly' in the database\n");
    printf("\t--clear_table \t\t [-b] \t\t\t The program deletes all records from tables 'cluster_usage_hourly', 'cluster_usage_daily', 'cluster_usage_monthly'\n");
    printf("\t--delete_old_records \t [-r] \t\t\t The program removes obsolete entries from tables. For 'cluster_usage_hourly' older than 2 months. For 'cluster_usage_daily' older than 3 years. For 'cluster_usage_monthly' older than 5 years.\n");
    printf("\t--off_write_day \t [-o] \t\t\t The program doesn't write the average load per day\n");
    printf("\t--off_write_month \t [-i] \t\t\t The program doesn't write the average load per month\n");
    printf("\t--help   \t\t [-h] \t\t\t Involved overview\n");
    printf("\nExample : \n");
    printf("\t%s -s tcp://localhost:5001 -t 10000\n", progname);
    printf("\nYou can run the program without parameters, then will be used default:\n");
    printf("\t--subsribe \t\t tcp://localhost:5001\n");
    printf("\t--time_calculate \t 30000\n");
    printf("\t--server \t\t localhost\n");
    printf("\t--user \t\t\t root\n");
    printf("\t--password \t\t ''\n");
    printf("\t--database \t\t slurm_acct\n");
    exit(1);
}

int main(int argc, char** argv)
{
    Subscriber* sub = new Subscriber();
    Keeper* keeper = new Keeper();
    Writer* writer = new Writer();

    int option_index = 0;
    int index;
    int action = 0;
    const char* short_options = "hs:t:cbe:u:p:d:oir";
    const struct option long_options[] = {
        { "help", no_argument, NULL, 'h' },
        { "subscribe", required_argument, NULL, 's' },
        { "time_calculate", required_argument, NULL, 't' },
        { "create_table", no_argument, NULL, 'c' },
        { "clear_table", no_argument, NULL, 'b' },
        { "server", no_argument, NULL, 'e' },
        { "user", no_argument, NULL, 'u' },
        { "password", no_argument, NULL, 'p' },
        { "database", no_argument, NULL, 'd' },
        { "off_write_day", no_argument, NULL, 'o' },
        { "off_write_month", no_argument, NULL, 'i' },
        { "delete_old_records", no_argument, NULL, 'r' },
        { NULL, 0, NULL, 0 }
    };

    while ((index = getopt_long(argc, argv, short_options, long_options, &option_index)) != -1) {
        switch (index) {
        case 'h': {
            help(argv[0]);
            break;
        }
        case 's': {
            sub->Set_Addr(zsys_sprintf(">%s", optarg));
            break;
        }
        case 't': {
            keeper->Set_Time_Calc(atoi(optarg));
            break;
        }
        case 'c': {
            action = 1;
            break;
        }
        case 'b': {
            action = 2;
            break;
        }
        case 'e': {
            writer->Set_Server(optarg);
            break;
        }
        case 'u': {
            writer->Set_User(optarg);
            break;
        }
        case 'p': {
            writer->Set_Password(optarg);
            break;
        }
        case 'd': {
            writer->Set_Database(optarg);
            break;
        }
        case 'o': {
            writer->Set_Off_Write_Day();
            break;
        }
        case 'i': {
            writer->Set_Off_Write_Month();
            break;
        }
        case 'r': {
            action = 3;
            break;
        }
        case '?': {
            printf("Use help:\n");
            printf("\t%s --help\n", argv[0]);
            return 0;
            break;
        }
        }
    }

    if (optind < argc) {
        printf("non-option ARGV-elements: ");
        while (optind < argc)
            printf("%s ", argv[optind++]);
        putchar('\n');
        printf("Use help:\n");
        printf("\t%s --help\n", argv[0]);
        return 0;
    }

    if (!writer->Connect_DB()) {
        delete sub;
        delete keeper;
        delete writer;
        return 1;
    }

    switch (action) {
    case 1: {
        writer->Create_Tables();
        return 0;
        break;
    }
    case 2: {
        writer->Clear_Tables();
        return 0;
        break;
    }

    case 3: {
        writer->Delete_Old_Records();
        printf("Old records are deleted\n");
        return 0;
        break;
    }
    }
    sub->Connect();

    while (!zsys_interrupted) {
        sub->Recv_Message();
        keeper->Insert_Message(sub->Get_Cluster_Json());
        if (keeper->Time_Write()) {
            writer->Write(keeper->Get_Map());
            keeper->Clear_Map();
        }
    }

    delete sub;
    delete keeper;
    delete writer;
    return 0;
}