#include <czmq.h>
#include <getopt.h>
#include <hlit-config.h>
#include <mysql.h>
#include <string>

void help(char* progname)
{
    printf("\n%s %d.%d.%d\n", HLITMON_NAME, HLITMON_VERSION_MAJOR,
        HLITMON_VERSION_MINOR, HLITMON_VERSION_PATCH);
    printf("\nUsage: %s <command>\n", progname);
    printf("\nWhere <command> is:\n");
    printf("\t--help     [-h]           \t Involved overview\n");
    printf("\t--subsribe [-s] <address>\t Address, where messages are coming\n");
    printf("\t--server \t\t [-e] <name> \t\t Server on which the database resides\n");
    printf("\t--user \t\t\t [-u] <name> \t\t The user on whose behalf program makes records in the database\n");
    printf("\t--password \t\t [-p] <password> \t The password of user on whose behalf program makes records in the database\n");
    printf("\t--database \t\t [-d] <name> \t\t The name of database to which program makes records\n");
    printf("\t--table \t\t [-t] <name> \t\t The name of table to which program makes records\n");
    
    printf("\nExample : \n");
    printf("\t%s -s tcp://localhost:5001\n", progname);
    printf("\nYou can run the program without parameters, then will be used default:\n");
    printf("\t--subsribe \t tcp://localhost:5001\n");
    printf("\t--server \t\t localhost\n");
    printf("\t--user \t\t\t root\n");
    printf("\t--password \t\t ''\n");
    printf("\t--database \t\t hlit_soft\n");
    printf("\t--table \t\t hlit_soft_table\n");
    exit(1);
}

int main(int argc, char** argv)
{
    char* adr_sub = strdup("@tcp://*:5002");
    std::string mysql_server = "localhost";
    std::string mysql_user = "root";
    std::string mysql_password = "";
    std::string mysql_database = "hlit_soft";
    std::string mysql_table = "soft_table";
    
    int option_index = 0;
    const char* short_options = "hs:e:u:p:d:t:";
    const struct option long_options[] = {
        { "help", no_argument, NULL, 'h' },
        { "subscribe", required_argument, NULL, 's' },
        { "server", no_argument, NULL, 'e' },
        { "user", no_argument, NULL, 'u' },
        { "password", no_argument, NULL, 'p' },
        { "database", no_argument, NULL, 'd' },
        { "table", no_argument, NULL, 't' },
        { NULL, 0, NULL, 0 }
    };

    int index;

    while ((index = getopt_long(argc, argv, short_options, long_options, &option_index)) != -1) {
        switch (index) {
        case 'h': {
            help(argv[0]);
            break;
        }
        case 's': {
            adr_sub = zsys_sprintf("@%s", optarg);
            break;
        }
        case 'e': {
            mysql_server = optarg;
            break;
        }
        case 'u': {
            mysql_user = optarg;
            break;
        }
        case 'p': {
            mysql_password = optarg;
            break;
        }
        case 'd': {
            mysql_database = optarg;
            break;
        }
        case 't': {
            mysql_table = optarg;
            break;
        }
        case '?': {
            printf("Use help:\n");
            printf("\t%s --help\n", argv[0]);
            return 0;
            break;
        }
        }
    }

    if (optind < argc) {
        printf("non-option ARGV-elements: ");
        while (optind < argc)
            printf("%s ", argv[optind++]);
        putchar('\n');
        printf("Use help:\n");
        printf("\t%s --help\n", argv[0]);
        return 0;
    }
    printf("Connecting to %s@%s:%s mysql database ...\n", mysql_user.data(), mysql_server.data(), mysql_database.data());
    
    MYSQL* writer = mysql_init(NULL);
    if (!mysql_real_connect(writer, mysql_server.data(), mysql_user.data(), mysql_password.data(), mysql_database.data(), 0, NULL, 0)) {
        fprintf(stderr, "%s\n", mysql_error(writer));
        return 2;
    }

    printf("Сonnection to %s@%s:%s database is established\n", mysql_user.data(), mysql_server.data(), mysql_database.data());

    zsock_t* sub = zsock_new_sub(adr_sub,"");
    assert(sub);

    printf("Client is connected to %s\n", &adr_sub[1]);

    while (!zsys_interrupted) {
        zmsg_t* msg = zmsg_recv(sub);
        if (!msg)
            break;

        if (mysql_ping(writer) > 0) {
            if (!mysql_real_connect(writer, mysql_server.data(), mysql_user.data(), mysql_password.data(), mysql_database.data(), 0, NULL, 0)) {
                fprintf(stderr, "%s\n", mysql_error(writer));
            }
        }

        char* group = zmsg_popstr(msg);
        char* name = zmsg_popstr(msg);
        char* version = zmsg_popstr(msg);
        char* hostname = zmsg_popstr(msg);
        char* username = zmsg_popstr(msg);
        char* timestamp = zmsg_popstr(msg);

        printf("Sending group=%s name=%s version=%s hostname=%s username=%s timestamp=%s ...\n", group, name, version, hostname, username, timestamp);
        char* sql_insert = zsys_sprintf("INSERT INTO %s (soft_group,soft_name,soft_version,hostname,username,timestamp) VALUES('%s', '%s', '%s','%s','%s', %s);", mysql_table.data(), group, name, version, hostname, username,timestamp);
        // printf("%s\n",sql_insert);
        if (mysql_query(writer, sql_insert)) {
            fprintf(stderr, "%s\n", mysql_error(writer));
        }
        free(sql_insert);

        free(group);
        free(name);
        free(version);
        free(hostname);
        free(username);
        free(timestamp);
        zmsg_destroy(&msg);
    }
    zsock_destroy(&sub);
    mysql_close(writer);

    free(adr_sub);

    return 0;
}
