#include <ctime>
#include <czmq.h>
#include <errno.h>
#include <fcntl.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <syslog.h>
#include <unistd.h>

#include "../sensor/HlitNodeSensor.h"

#include <hlit-config.h>

using namespace std;

void help(char* progname)
{
    printf("\n%s %d.%d.%d\n", HLITMON_NAME, HLITMON_VERSION_MAJOR,
        HLITMON_VERSION_MINOR, HLITMON_VERSION_PATCH);
    printf("\nUsage: %s <command>\n", progname);
    printf("\nWhere <command> is:\n");
    printf("\t--publish    [-p] <address> \t\t Address, where to send messages\n");
    printf("\t--help       [-h]  \t\t         Show this help\n");
    printf("\nExample : \n");
    printf("\t%s -p tcp://localhost:5000 hlit/intel/2017\n", progname);
    exit(1);
}
const char* getUserName();

int main(int argc, char* argv[])
{

    if (argc == 1)
        return 0;

    char* adr_pub = strdup(">tcp://localhost:5002");
    int timewait = 100;
    int option_index = 0;
    const char* short_options = "hp:t:";
    const struct option long_options[] = {
        { "help", no_argument, NULL, 'h' },
        { "publish", required_argument, NULL, 'p' },
        { "timewait", required_argument, NULL, 't' },
        { NULL, 0, NULL, 0 }
    };

    int index;

    while ((index = getopt_long(argc, argv, short_options, long_options, &option_index)) != -1) {
        switch (index) {
        case 'h': {
            help(argv[0]);
            break;
        }
        case 'p': {
            adr_pub = zsys_sprintf(">%s", optarg);
            break;
        }
        case 't': {
            timewait = atoi(optarg);
            break;
        }
        case '?': {
            printf("Use help:\n");
            printf("\t%s --help\n", argv[0]);
            return 0;
            break;
        }
        }
    }

    zsock_t* pub_snr = zsock_new_pub(adr_pub);
    assert(pub_snr);
    char* hostname = zsys_hostname();
    unsigned long int time_sec= time(NULL);

    zclock_sleep(timewait);

    if (optind < argc) {

        while (optind < argc) {

            char str[1024];
            strcpy(str, argv[optind++]);
            char* end_str;
            char* token = strtok_r(str, ":", &end_str);

            while (token != NULL) {
                zmsg_t* msg = zmsg_new();
                char* end_token;
                char* token2 = strtok_r(token, "/", &end_token);
                while (token2 != NULL) {
                    zmsg_addstrf(msg, "%s", token2);
                    token2 = strtok_r(NULL, "/", &end_token);
                }

                zmsg_addstrf(msg, "%s", hostname);
                zmsg_addstrf(msg, "%s", getUserName());
                zmsg_addstrf(msg, "%ld", time_sec);
                // zmsg_print(msg);
                zmsg_send(&msg, pub_snr);
                zmsg_destroy(&msg);

                token = strtok_r(NULL, ":", &end_str);
            }
        }
    }
    zclock_sleep(timewait);

    zsock_destroy(&pub_snr);
    free(hostname);
    free(adr_pub);

    return 0;
}

const char* getUserName()
{
    uid_t uid = geteuid();
    struct passwd* pw = getpwuid(uid);
    if (pw) {
        return pw->pw_name;
    }

    return "";
}