#include <czmq.h>
#include <getopt.h>
#include <hlit-config.h>

void help(char* progname)
{
    printf("\n%s %d.%d.%d\n", HLITMON_NAME, HLITMON_VERSION_MAJOR,
        HLITMON_VERSION_MINOR, HLITMON_VERSION_PATCH);
    printf("\nUsage: %s <command>\n", progname);
    printf("\nWhere <command> is:\n");
    printf("\t--subsribe [-s] <address>\t Address, where messages are coming\n");
    printf("\t--help     [-h]           \t Involved overview\n");
    printf("\nExample : \n");
    printf("\t%s -s tcp://localhost:5001\n", progname);
    printf("\nYou can run the program without parameters, then will be used default:\n");
    printf("\t--subsribe \t tcp://localhost:5001\n");
    exit(1);
}

int main(int argc, char** argv)
{
    char* adr_sub = strdup(">tcp://localhost:5001");
    int option_index = 0;
    const char* short_options = "hs:";
    const struct option long_options[] = {
        { "help", no_argument, NULL, 'h' },
        { "subscribe", required_argument, NULL, 's' },
        { NULL, 0, NULL, 0 }
    };

    int index;

    while ((index = getopt_long(argc, argv, short_options, long_options, &option_index)) != -1) {
        switch (index) {
        case 'h': {
            help(argv[0]);
            break;
        }
        case 's': {
            adr_sub = zsys_sprintf(">%s", optarg);
            break;
        }
        case '?': {
            printf("Use help:\n");
            printf("\t%s --help\n", argv[0]);
            return 0;
            break;
        }
        }
    }

    if (optind < argc) {
        printf("non-option ARGV-elements: ");
        while (optind < argc)
            printf("%s ", argv[optind++]);
        putchar('\n');
        printf("Use help:\n");
        printf("\t%s --help\n", argv[0]);
        return 0;
    }

    zsock_t* sub = zsock_new_sub(adr_sub, "");
    assert(sub);

    printf("Client is connected to %s\n", &adr_sub[1]);

    while (!zsys_interrupted) {
        zmsg_t* msg = zmsg_recv(sub);
        if (!msg)
            break;

        char* sub_str = zmsg_popstr(msg);
        char* cluster = zmsg_popstr(msg);
        char* json_str = zmsg_popstr(msg);

        printf("sub=%s cluster=%s json=%s\n", sub_str, cluster, json_str);
        free(sub_str);
        free(json_str);
        free(cluster);
        zmsg_destroy(&msg);
    }
    zsock_destroy(&sub);

    free(adr_sub);

    return 0;
}
